@@ -17,7 +17,7 @@
  */
 
 /*
- * $Id: netlib.c,v 1.11.2.2 2008/01/07 22:35:56 mtbishop Exp $
+ * $Id: netlib.c,v 1.11.2.4 2009/03/29 10:44:02 mtbishop Exp $
  */ 
 
 #include "config.h"
@@ -147,6 +147,7 @@ int udp_session(struct vtun_host *host)
      struct sockaddr_in saddr; 
      short port;
      int s,opt;
+     extern int is_rmt_fd_connected;
 
      if( (s=socket(AF_INET,SOCK_DGRAM,0))== -1 ){
         vtun_syslog(LOG_ERR,"Can't create socket");
@@ -190,10 +191,22 @@ int udp_session(struct vtun_host *host)
      }
 
      saddr.sin_port = port;
+
+     /* if the config says to delay the UDP connection, we wait for an
+	incoming packet and then force a connection back.  We need to
+	put this here because we need to keep that incoming triggering
+	packet and pass it back up the chain. */
+
+     if (VTUN_USE_NAT_HACK(host))
+     	is_rmt_fd_connected=0;
+	else {
      if( connect(s,(struct sockaddr *)&saddr,sizeof(saddr)) ){
         vtun_syslog(LOG_ERR,"Can't connect socket");
         return -1;
      }
+     is_rmt_fd_connected=1;
+	}
+     
      host->sopt.rport = htons(port);
 
      /* Close TCP socket and replace with UDP socket */	
